package gs.config.source

import cats.Applicative
import gs.config.ConfigKey
import java.util.UUID

/** In-memory implementation based on an immutable map.
  *
  * The raw value of the [[ConfigName]] is used for lookups (`toRawString()`).
  *
  * @param configs
  *   The configurations to provide.
  */
final class MemoryConfigSource[F[_]: Applicative](
  private val configs: Map[String, String]
) extends ConfigSource[F]:
  val id: UUID = UUID.randomUUID()

  override def getValue(
    key: ConfigKey[?]
  ): F[Option[String]] =
    Applicative[F].pure(
      configs
        .get(key.name.toRawString())
    )

  override lazy val name: String = s"in-memory-$id"
